@@ -10,7 +10,7 @@ use ross_config::filter::*;
 use ross_config::matcher::Matcher;
 use ross_config::producer::state::*;
 use ross_config::producer::*;
-use ross_config::Value;
+use ross_config::StateValue;
 use ross_protocol::event::event_code::*;
 
 use crate::tokenizer::{DataToken, KeywordToken, SymbolToken, Token, Tokenizer, TokenizerError};
@@ -24,6 +24,33 @@ macro_rules! prepare_variable {
     };
 }
 
+macro_rules! match_variable_or_value {
+    ($token_iterator:expr, $variable_map:expr) => {
+        match $token_iterator.next() {
+            Some(Token::Data(DataToken::Integer(value))) => Variable::Integer(*value),
+            Some(Token::Text(value)) => {
+                if let Some(variable) = $variable_map.get(value) {
+                    *variable
+                } else {
+                    return Err(ParserError::UndefinedVariable(value.clone()));
+                }
+            }
+            Some(token) => return Err(ParserError::UnexpectedToken(token.clone())),
+            None => return Err(ParserError::UnexpectedEndOfFile),
+        };
+    };
+}
+
+macro_rules! match_keyword_token {
+    ($token_iterator:expr, $keyword_token:path) => {
+        match $token_iterator.next() {
+            Some(Token::Keyword($keyword_token)) => {}
+            Some(token) => return Err(ParserError::UnexpectedToken(token.clone())),
+            None => return Err(ParserError::UnexpectedEndOfFile),
+        };
+    };
+}
+
 macro_rules! match_text_token {
     ($token_iterator:expr) => {
         match $token_iterator.next() {
@@ -69,7 +96,7 @@ enum Variable {
     Boolean(bool),
 }
 
-impl<'a> Parser {
+impl Parser {
     pub fn parse(text: &str) -> Result<Config, ParserError> {
         let mut initial_state = BTreeMap::new();
         let mut variable_map = BTreeMap::new();
@@ -82,9 +109,9 @@ impl<'a> Parser {
 
         while let Some(token) = token_iterator.next() {
             match token {
-                Token::Keyword(KeywordToken::Let) => {
+                Token::Keyword(KeywordToken::Store) => {
                     let mut state_name = String::new();
-                    let state = Self::parse_let_statement(&mut token_iterator, &mut state_name)?;
+                    let state = Self::parse_store_statement(&mut token_iterator, &mut state_name)?;
 
                     let mut state_index = 0;
 
@@ -98,7 +125,20 @@ impl<'a> Parser {
                     {
                         return Err(ParserError::DuplicateVariable);
                     }
-                }
+                },
+                Token::Keyword(KeywordToken::Const) => {
+                    let mut variable_name = String::new();
+                    let variable = Self::parse_const_statement(&mut token_iterator, &mut variable_name)?;
+
+                    if let Some(_) = variable_map.insert(variable_name, variable)
+                    {
+                        return Err(ParserError::DuplicateVariable);
+                    }
+                },
+                Token::Keyword(KeywordToken::Send) => {
+                    let event_processor = Self::parse_send_statement(&mut token_iterator, &variable_map)?;
+                    event_processors.push(event_processor);
+                },
                 Token::Keyword(KeywordToken::Do) => {
                     let event_processor =
                         Self::parse_do_statement(&mut token_iterator, &variable_map)?;
@@ -114,10 +154,10 @@ impl<'a> Parser {
         })
     }
 
-    fn parse_let_statement(
+    fn parse_store_statement(
         token_iterator: &mut Iter<Token>,
         state_name: &mut String,
-    ) -> Result<Value, ParserError> {
+    ) -> Result<StateValue, ParserError> {
         *state_name = match_text_token!(token_iterator);
 
         match_symbol_token!(token_iterator, SymbolToken::Colon);
@@ -136,15 +176,72 @@ impl<'a> Parser {
         match_symbol_token!(token_iterator, SymbolToken::Semicolon);
 
         let state = match state_type.as_str() {
-            "u8" => Value::U8(state_value.try_into()?),
-            "u16" => Value::U16(state_value.try_into()?),
-            "u32" => Value::U32(state_value.try_into()?),
+            "u8" => StateValue::U8(state_value.try_into()?),
+            "u16" => StateValue::U16(state_value.try_into()?),
+            "u32" => StateValue::U32(state_value.try_into()?),
+            "bool" => StateValue::Bool(state_value.try_into()?),
             _ => return Err(ParserError::TypeError),
         };
 
         Ok(state)
     }
 
+    fn parse_const_statement<'a>(
+        token_iterator: &mut Iter<Token>,
+        variable_name: &mut String,
+    ) -> Result<Variable, ParserError> {
+        *variable_name = match_text_token!(token_iterator);
+
+        match_symbol_token!(token_iterator, SymbolToken::EqualSign);
+
+        let variable = match token_iterator.next() {
+            Some(Token::Data(DataToken::Integer(value))) => Variable::Integer(*value),
+            Some(Token::Data(DataToken::Boolean(value))) => Variable::Boolean(*value),
+            Some(token) => return Err(ParserError::UnexpectedToken(token.clone())),
+            None => return Err(ParserError::UnexpectedEndOfFile),
+        };
+
+        match_symbol_token!(token_iterator, SymbolToken::Semicolon);
+
+        Ok(variable)
+    }
+
+    fn parse_send_statement(
+        token_iterator: &mut Iter<Token>,
+        variable_map: &BTreeMap<String, Variable>,
+    ) -> Result<EventProcessor, ParserError> {
+        let event_code: u16 = match_variable_or_value!(token_iterator, variable_map).try_into()?;
+
+        match_keyword_token!(token_iterator, KeywordToken::From);
+
+        let event_producer_address: u16 = match_variable_or_value!(token_iterator, variable_map).try_into()?;
+
+        match_keyword_token!(token_iterator, KeywordToken::To);
+
+        let receiver_address: u16 = match_variable_or_value!(token_iterator, variable_map).try_into()?;
+
+        match_symbol_token!(token_iterator, SymbolToken::Semicolon);
+
+        let matchers = vec![
+            Matcher {
+                extractor: Box::new(EventCodeExtractor::new()),
+                filter: Box::new(U16IsEqualFilter::new(event_code)),
+            },
+            Matcher {
+                extractor: Box::new(EventProducerAddressExtractor::new()),
+                filter: Box::new(U16IsEqualFilter::new(event_producer_address)),
+            },
+        ];
+        let extractor: Box<dyn Extractor> = Box::new(PacketExtractor::new());
+        let producer: Box<dyn Producer> = Box::new(PacketProducer::new(receiver_address));
+
+        Ok(EventProcessor {
+            matchers,
+            extractor,
+            producer,
+        })
+    }
+
     fn parse_do_statement(
         token_iterator: &mut Iter<Token>,
         variable_map: &BTreeMap<String, Variable>,
@@ -182,6 +279,21 @@ impl<'a> Parser {
         token_iterator: &mut Iter<Token>,
         variable_map: &BTreeMap<String, Variable>,
     ) -> Result<Matcher, ParserError> {
+        let mut sub_token_iterator = token_iterator.clone();
+
+        match sub_token_iterator.next() {
+            Some(Token::Keyword(KeywordToken::Event)) => {
+                match_keyword_token!(token_iterator, KeywordToken::Event);
+                return Self::parse_match_event_statement(token_iterator, variable_map)
+            },
+            Some(Token::Keyword(KeywordToken::Producer)) => {
+                match_keyword_token!(token_iterator, KeywordToken::Producer);
+                return Self::parse_match_producer_statement(token_iterator, variable_map)
+            },
+            _ => (),
+        }
+
+
         match_symbol_token!(token_iterator, SymbolToken::OpenBrace);
 
         let mut sub_token_iterator = token_iterator.clone();
@@ -214,6 +326,34 @@ impl<'a> Parser {
         Ok(Matcher { extractor, filter })
     }
 
+    fn parse_match_event_statement(
+        token_iterator: &mut Iter<Token>,
+        variable_map: &BTreeMap<String, Variable>,
+    ) -> Result<Matcher, ParserError> {
+        let event_code = match_variable_or_value!(token_iterator, variable_map);
+
+        match_symbol_token!(token_iterator, SymbolToken::Semicolon);
+
+        let extractor = Box::new(EventCodeExtractor::new());
+        let filter = Box::new(U16IsEqualFilter::new(event_code.try_into()?));
+
+        Ok(Matcher { extractor, filter })
+    }
+
+    fn parse_match_producer_statement(
+        token_iterator: &mut Iter<Token>,
+        variable_map: &BTreeMap<String, Variable>,
+    ) -> Result<Matcher, ParserError> {
+        let event_producer_address = match_variable_or_value!(token_iterator, variable_map);
+
+        match_symbol_token!(token_iterator, SymbolToken::Semicolon);
+
+        let extractor = Box::new(EventProducerAddressExtractor::new());
+        let filter = Box::new(U16IsEqualFilter::new(event_producer_address.try_into()?));
+
+        Ok(Matcher { extractor, filter })
+    }
+
     fn parse_fire_statement(
         token_iterator: &mut Iter<Token>,
         variable_map: &BTreeMap<String, Variable>,
@@ -276,6 +416,14 @@ impl<'a> Parser {
             return Ok(Box::new(EventCodeExtractor::new()));
         }
 
+        if extractor_type == "PacketExtractor" {
+            if arguments.len() != 0 {
+                return Err(ParserError::WrongArgumentCount);
+            }
+
+            return Ok(Box::new(PacketExtractor::new()));
+        }
+
         Err(ParserError::UnknownExtractor(extractor_type))
     }
 
@@ -397,6 +545,36 @@ impl<'a> Parser {
             )));
         }
 
+        if filter_type == "BoolIsEqualStateFilter" {
+            if arguments.len() != 2 {
+                return Err(ParserError::WrongArgumentCount);
+            }
+
+            return Ok(Box::new(BoolIsEqualStateFilter::new(
+                arguments[0]
+                    .try_into()
+                    .map_err(|_| ParserError::DataError)?,
+                arguments[1]
+                    .try_into()
+                    .map_err(|_| ParserError::DataError)?,
+            )));
+        }
+
+        if filter_type == "BoolSetStateFilter" {
+            if arguments.len() != 2 {
+                return Err(ParserError::WrongArgumentCount);
+            }
+
+            return Ok(Box::new(BoolSetStateFilter::new(
+                arguments[0]
+                    .try_into()
+                    .map_err(|_| ParserError::DataError)?,
+                arguments[1]
+                    .try_into()
+                    .map_err(|_| ParserError::DataError)?,
+            )));
+        }
+
         Err(ParserError::UnknownFilter(filter_type))
     }
 
@@ -454,6 +632,18 @@ impl<'a> Parser {
             )));
         }
 
+        if producer_type == "PacketProducer" {
+            if arguments.len() != 1 {
+                return Err(ParserError::WrongArgumentCount);
+            }
+
+            return Ok(Box::new(PacketProducer::new(
+                arguments[0]
+                    .try_into()
+                    .map_err(|_| ParserError::DataError)?,
+            )));
+        }
+
         Err(ParserError::UnknownProducer(producer_type))
     }
 
@@ -502,22 +692,6 @@ impl<'a> Parser {
     }
 
     fn prepare_variable_map(variable_map: &mut BTreeMap<String, Variable>) {
-        prepare_variable!(variable_map, NONE_EXTRACTOR_CODE);
-        prepare_variable!(variable_map, EVENT_CODE_EXTRACTOR_CODE);
-
-        prepare_variable!(variable_map, U8_INCREMENT_STATE_FILTER);
-        prepare_variable!(variable_map, U16_IS_EQUAL_FILTER_CODE);
-        prepare_variable!(variable_map, U32_IS_EQUAL_STATE_FILTER_CODE);
-        prepare_variable!(variable_map, U32_INCREMENT_STATE_FILTER_CODE);
-        prepare_variable!(variable_map, U32_SET_STATE_FILTER_CODE);
-        prepare_variable!(variable_map, FLIP_FLOP_FILTER_CODE);
-        prepare_variable!(variable_map, COUNT_FILTER_CODE);
-        prepare_variable!(variable_map, COUNT_STATE_FILTER_CODE);
-
-        prepare_variable!(variable_map, NONE_PRODUCER_CODE);
-        prepare_variable!(variable_map, BCM_CHANGE_BRIGHTNESS_PRODUCER_CODE);
-        prepare_variable!(variable_map, BCM_CHANGE_BRIGHTNESS_STATE_PRODUCER_CODE);
-
         prepare_variable!(variable_map, BOOTLOADER_HELLO_EVENT_CODE);
         prepare_variable!(variable_map, PROGRAMMER_HELLO_EVENT_CODE);
         prepare_variable!(variable_map, PROGRAMMER_START_FIRMWARE_UPGRADE_EVENT_CODE);
@@ -529,6 +703,7 @@ impl<'a> Parser {
         prepare_variable!(variable_map, BUTTON_RELEASED_EVENT_CODE);
         prepare_variable!(variable_map, INTERNAL_SYSTEM_TICK_EVENT_CODE);
         prepare_variable!(variable_map, PROGRAMMER_START_CONFIG_UPGRADE_EVENT_CODE);
+        prepare_variable!(variable_map, PROGRAMMER_SET_DEVICE_ADDRESS_EVENT_CODE);
     }
 }
 
